package concurrency2;

import java.util.Random;

/**
 * Inserts integers into an array shared with other Runnable objects.
 * 
 * @author Prof. Carl B. Struck
 * @version Copyright (c) SCCC Spring 2008
 */
public class ArrayWriter implements Runnable 
{
    private final SimpleArray sharedSimpleArray;
    private final int startValue;
    private final Random randomGenerator;

    /**
     * Initializes the starting value for this thread and the array shared 
     * between the threads.
     * 
     * @param startValue the starting value for this thread
     * @param sharedSimpleArray the array shared between the threads
     */
    public ArrayWriter(int startValue, SimpleArray sharedSimpleArray) 
    {
        this.startValue = startValue;
        this.sharedSimpleArray = sharedSimpleArray;
        
        randomGenerator = new Random();
    }

    /**
     * Adds an element to the shared array. Puts the thread to sleep 
     * for between zero (0) to 499 milliseconds before each attempt to
     * add another element.
     */
    @Override
    public void run() 
    {
        for (int value = startValue; 
             value < (startValue + sharedSimpleArray.getLength() / 2 ); 
             value++)
        {
            try
            {
                Thread.sleep( randomGenerator.nextInt(500) );
            }

            catch (InterruptedException ex)
            {
                System.out.printf("%n%sterminated prematurely due to interruption%n", 
                    Thread.currentThread().getName() );
                Thread.currentThread().interrupt(); // re-interrupt the thread 
            }
            
            sharedSimpleArray.add(value);
        }
    }
}
