package concurrency2;

import java.util.Arrays;

/**
 * A class that manages an integer array to be shared by multiple threads 
 * with synchronization.
 * 
 * @author Prof. Carl B. Struck
 * @version Copyright (c) SCCC Spring 2008
 */
public class SimpleArray 
{
    private final int array[]; // The shared integer array
    private int index;    // The index of next element to be written

    /**
     * Constructs a SimpleArray object of type <code>int</code> of a specified
     * size and initializes the 'index' to zero (0).
     * 
     * @param size the size of the array
     * @throws IllegalArgumentException if the size of the array is not a positive number
     */
    public SimpleArray(int size) throws IllegalArgumentException
    {
        if ( (size % 2) != 0)
        {
            throw new IllegalArgumentException("Array size must be a positive number");
        }
        
        array = new int[size];
        index = 0;
    }

    /**
     * Adds a value to an element of the shared array. The method is synchronized 
     * so the next task must wait until the current method is finished with the 
     * method and releases the lock.
     * 
     * @param value the value added to the shared array
     */
    public synchronized void add(int value)
    {
        array[index] = value;

        System.out.printf("%s wrote %d to element %d%n%n", 
                Thread.currentThread().getName(), value, index);

        index++;
    }
    
    /**
     * Returns the size of the array.
     * 
     * @return the size of the array
     */
    public int getLength()
    {
        return array.length;
    }

    /**
     * Returns the contents of the array as a String.
     * 
     * @return the array values
     */
    @Override
    public String toString()
    {
        return Arrays.toString(array);
    }
}
